

/* 
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Benjamin Grauer
   co-programmer: ...
*/


#include "light.h"

#include "glincl.h"

using namespace std;


/**
   \brief standard constructor for a Light
*/
Light::Light () 
{
  this->init();
}

Light::Light(Vector position)
{
  this->init();
  this->setPosition(position);
}

/**
   \brief standard deconstructor
   \todo this deconstructor is not jet implemented - do it

*/
Light::~Light () 
{
  glDisable(GL_LIGHTING);
  glDisable(GL_LIGHT0);
}

/**
   \brief initializes a new Light with default values, and enables GL_LIGHTING
*/
void Light::init()
{
  this->setClassName ("Light");
  glEnable (GL_LIGHTING);
  glEnable (GL_LIGHT0);
  
  this->setPosition(Vector(0.0, 0.0, 0.0));
  this->setDiffuseColor(1.0, 1.0, 1.0);
  this->setSpecularColor(1.0, 1.0, 1.0);
  //  lmodelAmbient[] = {.1, .1, .1, 1.0};
  
}

// set Attributes
/**
   \brief Sets a Position for the Light.
   \param position The new position of the Light.
   \todo patrick: is it ok to set a Light Position even if it is derived from p_node??
*/
void Light::setPosition(Vector position)
{
  this->lightPosition[0] = position.x;
  this->lightPosition[1] = position.y;
  this->lightPosition[2] = position.z;
  this->lightPosition[3] = 0.0;

  glLightfv (GL_LIGHT0, GL_POSITION, lightPosition);

}

/**
   \brief sets an emitting Diffuse color for the Light
   \param r red
   \param g green
   \param b blue
*/
void Light::setDiffuseColor(GLfloat r, GLfloat g, GLfloat b)
{
  this->diffuseColor[0] = r;
  this->diffuseColor[1] = g;
  this->diffuseColor[2] = b;
  this->diffuseColor[3] = 1.0;

  glLightfv (GL_LIGHT0, GL_DIFFUSE, diffuseColor);
}


/**
   \brief sets an emitting Ambient color for the Light
   \param r red
   \param g green
   \param b blue
*/
void Light::setSpecularColor(GLfloat r, GLfloat g, GLfloat b)
{
  this->specularColor[0] = r;
  this->specularColor[1] = g;
  this->specularColor[2] = b;
  this->specularColor[3] = 1.0;

  glLightfv (GL_LIGHT0, GL_SPECULAR, specularColor);
}

// get Attributes

/**
   \returns the Position of the Light
*/
Vector Light::getPosition(void)
{
  Vector tmpPosition(lightPosition[0], lightPosition[1], lightPosition[2]);
  return tmpPosition;
}
